package cz.muni.fi.exercise;

import cz.muni.fi.processor.XMLPrinter;
import cz.muni.fi.wicket.BasePage;
import cz.muni.fi.wicket.SignInSession;
import org.apache.wicket.MarkupContainer;
import org.apache.wicket.Session;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.model.PropertyModel;

/**
 * Abstract class for Page with exercise
 *
 * @author Jiří Krejčí
 */
public abstract class AExercisePage extends BasePage implements java.io.Serializable {

    protected XMLPrinter printer;
    protected String title;

    public AExercisePage() {
        super();
        printer = new XMLPrinter();

        /*
         * Lable for page title
         * ex. Schéma
         */
        Label titleLabel = new Label("title", new PropertyModel(this, "title") {
            @Override
            public Object getObject() {
                return (String) title;
            }
        });
        titleLabel.setEscapeModelStrings(false);
        add(titleLabel);

        /*
         * Label for task
         * ex. Napište XML Schéma pro zadané XML
         */
        Label taskLabel = new Label("task", new PropertyModel(this, "task") {
            @Override
            public Object getObject() {
                return (String) getExercise().getTask();
            }
        });
        taskLabel.setEscapeModelStrings(false);
        add(taskLabel);

        /*
         * Label for XML in task
         */
        Label taskXMLLabel = new Label("taskXML", new PropertyModel(this, "taskXML") {
            @Override
            public Object getObject() {
                return (String) printer.getEscapedXML(getExercise().getTaskXML());
            }
        });
        taskXMLLabel.setEscapeModelStrings(false);
        add(taskXMLLabel);


        MarkupContainer resultContainer = new MarkupContainer("resultContainer") {
            @Override
            public boolean isVisible() {
                SignInSession session = (SignInSession) Session.get();

                return !(session == null || session.getUser() == null);
            }
        };

        /*
         * Label for result message
         * ex. Váš výsledek je správně. Odpovídá očekávanému výsledku
         */
        Label resultLabel = new Label("result", new PropertyModel(this, "result") {
            @Override
            public Object getObject() {
                return (String) printer.getReplacedNewLine(getExercise().getResult());
            }
        });
        resultLabel.setEscapeModelStrings(false);
        resultContainer.add(resultLabel);

        /*
         * Label for XML in result
         */
        Label resultXMLLabel = new Label("resultXML", new PropertyModel(this, "resultXML") {
            @Override
            public Object getObject() {
                return (String) printer.getEscapedXML(getExercise().getResultXML());
            }
        });
        resultXMLLabel.setEscapeModelStrings(false);
        resultContainer.add(resultXMLLabel);

        add(resultContainer);

        /*
         * Label for result message
         * ex. Váš výsledek je správně. Odpovídá očekávanému výsledku
         */
        Label expectedResultLabel = new Label("expectedResult", new PropertyModel(this, "expectedResult") {
            @Override
            public Object getObject() {
                return (String) getExercise().getExpectedResult();
            }
        });
        expectedResultLabel.setEscapeModelStrings(false);
        add(expectedResultLabel);

        /*
         * Label for XML in result
         */
        Label expectedResultXMLLabel = new Label("expectedResultXML", new PropertyModel(this, "expectedResultXML") {
            @Override
            public Object getObject() {
                return (String) printer.getEscapedXML(getExercise().getExpectedResultXML());
            }
        });
        expectedResultXMLLabel.setEscapeModelStrings(false);
        add(expectedResultXMLLabel);


    }

    public String getTitle() {
        return title;
    }

    public void setTitle(String title) {
        this.title = title;
    }

    public abstract Exercise getExercise();
}