package cz.muni.fi.exercise;

import cz.muni.fi.processor.AProcessor;
import cz.muni.fi.processor.DTDProcessor;
import cz.muni.fi.wicket.SignInSession;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.apache.wicket.Session;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.model.Model;

/**
 * Class for Exercise Forms
 * 
 * @author Franta
 */
public class ExerciseForm extends Form {

    private TextArea input;
    private Exercise exercise;
    private AProcessor processor;

    public ExerciseForm(String id, AProcessor processor, Exercise exercise) {
        super(id);
        this.processor = processor;
        this.exercise = exercise;

        input = new TextArea("input", new Model());
        input.setModelObject((String) this.exercise.getInputXML());
        add(input);

        SignInSession session = (SignInSession) Session.get();
        if (session == null || session.getUser() == null) {
            setVisible(false);
        } else {
            setVisible(true);
        }
    }

    @Override
    protected void onSubmit() {
        try {

            processor.setInputXML((String) getInput().getModelObject());

            processor.validate();

            getExercise().setResultXML(processor.getResultXML());

            if (processor.isValidationCorrect()) {
                getExercise().setResult("<div class=\"done\">" + processor.getInfoMessage() + "</div>");
            } else {
                getExercise().setResult("<div class=\"error\">" + processor.getErrorMessage() + "</div>");
            }
            
            UserExerciseProtocolManager exerciseProtocolManager = new UserExerciseProtocolManager();
            exerciseProtocolManager.addProtocol(
                    new UserExerciseId(((SignInSession) Session.get()).getUser().getID(), exercise.getID()), 
                    (String) getInput().getModelObject(), 
                    processor.isValidationCorrect());

        } catch (Exception ex) {
            Logger.getLogger(DTDProcessor.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    public TextArea getInput() {
        return input;
    }

    public Exercise getExercise() {
        return exercise;
    }
}
