package cz.muni.fi.exercise;

import cz.muni.fi.hibernate.Database;
import java.util.List;

/**
 * Manager that controls Exercises
 *
 * @author Jiří Krejčí
 */
public class ExerciseManager implements java.io.Serializable {

    Database d;

    public ExerciseManager() {

        d = new Database();

    }

    /**
     * Method that returns list of all exercises
     *
     * @return List of all exercises
     */
    public List<Exercise> listAllExercises() {

        return listAllExercises("%");

    }

    /**
     * Method that returns list of exercieces with specified type
     *
     * @param lessonType Type of lesson to be listed
     * @return List of all exercises with specified type
     */
    public List<Exercise> listAllExercises(String lessonType) {

        return d.listAllExercises(lessonType);

    }

    /**
     * Method that gets exercises with specifed id.
     *
     * @param exerciseId Integer value of exercise
     * @return Exercise with specifed id.
     */
    public Exercise getExercise(Integer exerciseId) {

        return d.getExercise(exerciseId);

    }

    /**
     * Method that adds new Exercise into database.
     *
     * @param task Text part of the task
     * @param taskXML XML part of the task
     * @param inputXML Partial solution of the task
     * @param expectedResult Text part of the expected solution
     * @param expectedResultXML XML part of the expected solution
     * @param type Type of exercise
     * @param showExpectedResult String value that specifies if the sample
     * solution will be shown or not
     * @param outputXML XML part of generated solution
     */
    public void addExercise(String task, String taskXML,
            String inputXML, String expectedResult, String expectedResultXML,
            String type, String showExpectedResult, String outputXML) {

        Exercise exercise = new Exercise();
        exercise.setTask(task);
        exercise.setTaskXML(taskXML);
        exercise.setInputXML(inputXML);
        exercise.setExpectedResult(expectedResult);
        exercise.setExpectedResultXML(expectedResultXML);
        exercise.setExericseType(type);

        if (outputXML == null) {
            exercise.setOutputXML("");
        } else {
            exercise.setOutputXML(outputXML);
        }

        d.addExercise(exercise);

    }
}