package cz.muni.fi.exercise;

import cz.muni.fi.hibernate.Database;
import cz.muni.fi.user.User;
import java.util.List;

/**
 * Manager that controls UserExerciseProtocol
 *
 * @author Franta
 */
public class UserExerciseProtocolManager {

    Database d;

    public UserExerciseProtocolManager() {

        d = new Database();
    }

    /**
     * Method that adds new protocol.
     *
     * @param protocolId Protocol id composed by user id and exercise id
     * @param result Result produced by user
     * @param succeded Boolean value that says if the exercise is solution is
     * correct or not
     */
    public void addProtocol(UserExerciseId protocolId, String result, boolean succeded) {

        UserExerciseProtocol protocol = new UserExerciseProtocol();

        protocol.setUserExerciseId(protocolId);
        protocol.setResult(result);
        if (succeded) {
            protocol.setSucceeded("yes");
        } else {
            protocol.setSucceeded("no");
        }

        d.addUserExerciseProtocol(protocol);

    }

    /**
     * Method that gets protocol with specifed id.
     *
     * @param protocolId id that specifed the protocol
     * @return Protocol with specified id
     */
    public UserExerciseProtocol getProtocol(int protocolId) {

        return d.getUserExerciseProtocol(protocolId);

    }

    /**
     * Methot that lists all protocols of specified exercise for specifed user.
     *
     * @param user User who passed the exercise
     * @param exercise Passed exercise
     * @return List of all protocols of specifed exercise for specified user
     */
    public List<UserExerciseProtocol> listProtocols(User user, Exercise exercise) {

        return d.listUserExerciseProtocols(user, exercise);

    }

    /**
     * Method that update protocol comment. Requirs updated protocol and saves
     * new comment into database.
     *
     * @param exerciseProtocol Protocol to by updated
     */
    public void updateProtocol(UserExerciseProtocol exerciseProtocol) {

        d.saveComment(exerciseProtocol.getID(), exerciseProtocol.getComment());
    }
}
