package cz.muni.fi.hibernate;

/**
 * Class that controls interaction with database.
 *
 * @author Jiří Krejčí
 */
import cz.muni.fi.exercise.Exercise;
import cz.muni.fi.exercise.UserExerciseProtocol;
import cz.muni.fi.lesson.Lesson;
import cz.muni.fi.user.RegistrationPeriod;
import cz.muni.fi.user.Tag;
import cz.muni.fi.user.User;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.SessionScoped;
import org.apache.wicket.authroles.authorization.strategies.role.Roles;
import org.hibernate.Query;
import org.hibernate.Session;

@ManagedBean
@SessionScoped
public class Database {

    private Session session;

    /**
     * Method that adds new exercise
     *
     * @param exercise Exercise to be add
     */
    public void addExercise(Exercise exercise) {
        session = HibernateUtil.getSessionFactory().openSession();
        session.beginTransaction();
        session.save(exercise);
        session.getTransaction().commit();
        session.flush();
        session.close();
    }

    /**
     * Method that list all exercieses with specified type
     *
     * @param exerciseType Type of exercise
     * @return List of all exercises with specified type
     */
    public List<Exercise> listAllExercises(String exerciseType) {

        session = HibernateUtil.getSessionFactory().openSession();
        session.beginTransaction();
        Query q = session.createSQLQuery("SELECT * FROM `exercise` "
                + "WHERE `exercise_type` LIKE :exerciseType ;").addEntity(Exercise.class);
        q.setParameter("exerciseType", exerciseType);
        List<Exercise> list = q.list();

        session.getTransaction().commit();

        session.flush();
        session.close();

        return list;

    }

    /**
     * Method that returns exercise with specified id.
     *
     * @param id Integer value of exercise id
     * @return Exercise with specified id
     */
    public Exercise getExercise(Integer id) {

        session = HibernateUtil.getSessionFactory().openSession();
        session.beginTransaction();
        Query q = session.createSQLQuery("SELECT * FROM `exercise` "
                + "WHERE `exercise_id` = :exerciseid ;").addEntity(Exercise.class);
        q.setParameter("exerciseid", id);
        List<Exercise> list = q.list();
        session.getTransaction().commit();

        Exercise exercise = new Exercise();

        for (Exercise e : list) {
            exercise = e;
        }

        session.flush();
        session.close();

        return exercise;

    }

    /**
     * Method that gets lesson with specified type.
     *
     * @param lessonType Type of lesson
     * @return Lesson with specified type
     */
    public Lesson getLesson(String lessonType) {

        session = HibernateUtil.getSessionFactory().openSession();
        session.beginTransaction();
        Query q = session.createSQLQuery("SELECT * FROM `lesson` "
                + "WHERE `lesson_type`= :lessonType ;").addEntity(Lesson.class);
        q.setParameter("lessonType", lessonType);
        List<Lesson> list = q.list();
        session.getTransaction().commit();

        Lesson lesson = new Lesson();

        for (Lesson l : list) {
            lesson = l;
        }

        session.flush();
        session.close();

        return lesson;

    }

    /**
     * Method that gets lesson with specified id.
     *
     * @param lessonId int value of lesson
     * @return Lesson with specified id
     */
    public Lesson getLesson(int lessonId) {

        session = HibernateUtil.getSessionFactory().openSession();
        session.beginTransaction();
        Lesson l = (Lesson) session.get(Lesson.class, lessonId);
        session.getTransaction().commit();
        session.flush();
        session.close();

        return l;

    }

    /**
     * Methot that updates lesson.
     *
     * @param lesson Lesson to by updated
     */
    public void updateLesson(Lesson lesson) {
        session = HibernateUtil.getSessionFactory().openSession();
        session.beginTransaction();
        session.update(lesson);
        session.getTransaction().commit();
        session.flush();
        session.close();

    }

    /**
     * Method that lists all lessons.
     *
     * @return List of all lesons
     */
    public List<Lesson> listAllLessons() {

        session = HibernateUtil.getSessionFactory().openSession();
        session.beginTransaction();
        Query q = session.createSQLQuery("SELECT * FROM `lesson`;").addEntity(Lesson.class);
        List<Lesson> list = q.list();

        session.getTransaction().commit();

        session.flush();
        session.close();

        return list;

    }

    /**
     * Method that returns user with specified username
     *
     * @param username User username
     * @return User with specified username
     */
    public User getUser(String username) {

        session = HibernateUtil.getSessionFactory().openSession();
        session.beginTransaction();
        Query q = session.createSQLQuery("SELECT * FROM `user` "
                + "WHERE `username`= :username ;").addEntity(User.class);
        q.setParameter("username", username);
        List<User> list = q.list();
        session.getTransaction().commit();

        User user = new User();

        for (User u : list) {
            user = u;
        }

        session.flush();
        session.close();

        return user;

    }

    /**
     * Method that returns user with specified username
     *
     * @param int Int value of user id
     * @return User with specified id
     */
    public User getUser(int id) {

        session = HibernateUtil.getSessionFactory().openSession();
        session.beginTransaction();
        Query q = session.createSQLQuery("SELECT * FROM `user` "
                + "WHERE `user_id` = :userid ;").addEntity(User.class);
        q.setParameter("userid", id);
        List<User> list = q.list();
        session.getTransaction().commit();

        User user = new User();

        for (User u : list) {
            user = u;
        }

        session.flush();
        session.close();

        return user;

    }

    /**
     * Method that adds new user
     *
     * @param user User to by add
     */
    public void addUser(User user) {
        session = HibernateUtil.getSessionFactory().openSession();
        session.beginTransaction();
        session.save(user);
        session.getTransaction().commit();
        session.flush();
        session.close();
    }

    /**
     * Method that lists all users.
     *
     * @return List of all users
     */
    public List<User> listAllUsers() {

        session = HibernateUtil.getSessionFactory().openSession();
        session.beginTransaction();
        Query q = session.createSQLQuery("SELECT * FROM `user`").addEntity(User.class);
        List<User> list = q.list();

        session.getTransaction().commit();

        session.flush();
        session.close();

        return list;

    }

    /**
     * Method that lists alle users with specified Tag.
     *
     * @param tag Tag to filter users
     * @return Returns users with specified Tag
     */
    public List<User> listAllUsers(Tag tag) {

        session = HibernateUtil.getSessionFactory().openSession();
        session.beginTransaction();
        Query q = session.createSQLQuery("SELECT `user`.`user_id`, `user`.`username`, `user`.`uco`, `user`.`password`, `user`.`firstname`, `user`.`lastname`, `user`.`role` "
                + "FROM `user` JOIN `user_usertags` JOIN `usertags` "
                + "ON `user`.`user_id` = `user_usertags`.`user_id` AND `usertags`.`usertag_id` = `user_usertags`.`usertag_id` "
                + "WHERE `usertags`.`name` = :usertag ;").addEntity(User.class);
        q.setParameter("usertag", tag.getName());
        List<User> list = q.list();

        session.getTransaction().commit();

        session.flush();
        session.close();

        return list;

    }

    /**
     * Method that lists alle users with specified role.
     *
     * @param r Role to filter users
     * @return Returns users with specified role
     */
    public List<User> listAllUsers(Roles r) {

        session = HibernateUtil.getSessionFactory().openSession();
        session.beginTransaction();
        Query q = session.createSQLQuery("SELECT `user`.`user_id`, `user`.`username`, `user`.`uco`, `user`.`password`, `user`.`firstname`, `user`.`lastname`, `user`.`role` "
                + "FROM `user` "
                + "WHERE `user`.`role` = :role ;").addEntity(User.class);
        q.setParameter("role", r.toString());
        List<User> list = q.list();

        session.getTransaction().commit();

        session.flush();
        session.close();

        return list;

    }

    /**
     * Method that returns all users with specified id and specified role.
     *
     * @param tag Tag to filter users
     * @param r Role to filter users
     * @return List of all users with specified id and specified role
     */
    public List<User> listAllUsers(Tag tag, Roles r) {

        session = HibernateUtil.getSessionFactory().openSession();
        session.beginTransaction();
        Query q = session.createSQLQuery("SELECT `user`.`user_id`, `user`.`username`, `user`.`uco`, `user`.`password`, `user`.`firstname`, `user`.`lastname`, `user`.`role` "
                + "FROM `user` JOIN `user_usertags` JOIN `usertags` "
                + "ON `user`.`user_id` = `user_usertags`.`user_id` AND `usertags`.`usertag_id` = `user_usertags`.`usertag_id`"
                + "WHERE `usertags`.`name` = :usertag AND `user`.`role` = :role ;").addEntity(User.class);
        q.setParameter("usertag", tag.getName());
        q.setParameter("role", r.toString());
        List<User> list = q.list();

        session.getTransaction().commit();

        session.flush();
        session.close();

        return list;

    }

    /**
     * Method that adds new Tag.
     *
     * @param tag Tag to be add
     */
    public void addTag(Tag tag) {
        session = HibernateUtil.getSessionFactory().openSession();
        session.beginTransaction();
        session.save(tag);
        session.getTransaction().commit();
        session.flush();
        session.close();
    }

    /**
     * Method that gets tag with specifed tag name.
     *
     * @param name Tag name to be find
     * @return Tag with specified tag name
     */
    public Tag getTag(String name) {
        session = HibernateUtil.getSessionFactory().openSession();
        session.beginTransaction();
        Query q = session.createSQLQuery("SELECT * FROM `usertags` "
                + "WHERE `name`= :name ;").addEntity(Tag.class);
        q.setParameter("name", name);
        List<Tag> list = q.list();
        session.getTransaction().commit();

        Tag tag = new Tag();

        for (Tag u : list) {
            tag = u;
        }

        session.flush();
        session.close();

        return tag;
    }

    /**
     * Method that lists all tags of specified user.
     *
     * @param user User's tags to be search
     * @return List of tags of specifed user
     */
    public List<Tag> listUserTags(User user) {

        session = HibernateUtil.getSessionFactory().openSession();
        session.beginTransaction();

        Query q = session.createSQLQuery("SELECT `usertags`.`usertag_id` , `name` "
                + "FROM `user` JOIN `user_usertags` JOIN `usertags` "
                + "WHERE `user`.`user_id` = `user_usertags`.`user_id` "
                + "AND `user_usertags`.`usertag_id` = `usertags`.`usertag_id`"
                + "AND `user`.`user_id` = :userId ;").addEntity(Tag.class);
        q.setParameter("userId", String.valueOf(user.getID()));
        List<Tag> list = q.list();

        session.getTransaction().commit();

        session.flush();
        session.close();

        return list;

    }

    /**
     * Method that lists all tags.
     *
     * @return List of all tags
     */
    public List<Tag> listAllTags() {

        session = HibernateUtil.getSessionFactory().openSession();
        session.beginTransaction();

        Query q = session.createSQLQuery("SELECT * FROM `usertags`;").addEntity(Tag.class);
        List<Tag> list = q.list();

        session.getTransaction().commit();

        session.flush();
        session.close();

        return list;

    }

    /**
     * Method that adds new UserTag
     *
     * @param u Uset to be tagged
     * @param t Tag to be added to user
     */
    public void addUserTag(User u, Tag t) {
        session = HibernateUtil.getSessionFactory().openSession();
        session.beginTransaction();
        Query q = session.createSQLQuery("INSERT INTO `user_usertags` (`user_id`, `usertag_id`) "
                + "VALUES ( :userId , :tagId );");
        q.setParameter("userId", String.valueOf(u.getID()));
        q.setParameter("tagId", String.valueOf(t.getID()));
        q.executeUpdate();
        session.getTransaction().commit();
        session.flush();
        session.close();
    }

    /**
     * Method that lists all protocols of specified exercise for specified user.
     *
     * @param user User's protocol to be serached
     * @param exercise Exercise's protocols to be searched
     * @return List of all protocols of specified exercise for specified user
     */
    public List<UserExerciseProtocol> listUserExerciseProtocols(User user, Exercise exercise) {

        session = HibernateUtil.getSessionFactory().openSession();
        session.beginTransaction();

        Query q = session.createSQLQuery("SELECT * FROM `user_exercise` "
                + "WHERE `user_id` = :userid AND `exercise_id` = :exerciseid ORDER BY done_timestamp DESC ;").addEntity(UserExerciseProtocol.class);
        q.setParameter("userid", String.valueOf(user.getID()));
        q.setParameter("exerciseid", String.valueOf(exercise.getID()));
        List<UserExerciseProtocol> list = q.list();

        session.getTransaction().commit();

        session.flush();
        session.close();

        return list;

    }

    /**
     * Method that adds new protocol
     * 
     * @param ue Protocol to be add
     */
    public void addUserExerciseProtocol(UserExerciseProtocol ue) {
        session = HibernateUtil.getSessionFactory().openSession();
        session.beginTransaction();
        session.save(ue);
        session.getTransaction().commit();
        session.flush();
        session.close();
    }

    /**
     * Method that gets protocol with specified id.
     *
     * @param id Int value of protocol
     * @return Protocol with specified id
     */
    public UserExerciseProtocol getUserExerciseProtocol(int id) {

        session = HibernateUtil.getSessionFactory().openSession();
        session.beginTransaction();

        Query q = session.createSQLQuery("SELECT * FROM `user_exercise` "
                + "WHERE `user_exercise_id` = :id ;").addEntity(UserExerciseProtocol.class);
        q.setParameter("id", id);
        List<UserExerciseProtocol> list = q.list();
        session.getTransaction().commit();

        UserExerciseProtocol userExercise = new UserExerciseProtocol();

        for (UserExerciseProtocol r : list) {
            userExercise = r;
        }

        session.flush();
        session.close();

        return userExercise;

    }

    /**
     * Method that updates protocol comment
     *
     * @param id Int value of protocol to be updated
     * @param comment Value of comment to be saved
     */
    public void saveComment(int id, String comment) {
        session = HibernateUtil.getSessionFactory().openSession();
        session.beginTransaction();
        Query q = session.createSQLQuery("UPDATE `user_exercise` "
                + "SET `comment` = :comment WHERE `user_exercise`.`user_exercise_id` = :id ;");
        q.setParameter("id", id);
        q.setParameter("comment", comment);
        q.executeUpdate();
        session.getTransaction().commit();
        session.flush();
        session.close();
    }

    /**
     * Method that gets Registration period with specified password.
     *
     * @param password Password to be searched
     * @return Registration priod with specified password
     */
    public RegistrationPeriod getRegistration(String password) {

        session = HibernateUtil.getSessionFactory().openSession();
        session.beginTransaction();

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Date date = new Date();

        Query q = session.createSQLQuery("SELECT * FROM `registration` "
                + "WHERE `from_timestamp` <= :from_timestamp "
                + "AND `to_timestamp` >= :to_timestamp "
                + "AND `password`= :password ;").addEntity(RegistrationPeriod.class);
        q.setParameter("from_timestamp", sdf.format(date));
        q.setParameter("to_timestamp", sdf.format(date));
        q.setParameter("password", password);
        List<RegistrationPeriod> list = q.list();
        session.getTransaction().commit();

        RegistrationPeriod reg = new RegistrationPeriod();

        for (RegistrationPeriod r : list) {
            reg = r;
        }

        session.flush();
        session.close();

        return reg;

    }

    /**
     * Method that adds new Registration period
     *
     * @param registrationPeriod Registration period to be add
     */
    public void addRegistrationPeriod(RegistrationPeriod registrationPeriod) {
        session = HibernateUtil.getSessionFactory().openSession();
        session.beginTransaction();
        session.save(registrationPeriod);
        session.getTransaction().commit();
        session.flush();
        session.close();
    }

    /**
     * Method that lists all registraiotn periods
     *
     * @return List of all registraiton periods
     */
    public List<RegistrationPeriod> listAllRegistrationPeriod() {

        session = HibernateUtil.getSessionFactory().openSession();
        session.beginTransaction();
        Query q = session.createSQLQuery("SELECT * FROM `registration`").addEntity(RegistrationPeriod.class);
        List<RegistrationPeriod> list = q.list();

        session.getTransaction().commit();

        session.flush();
        session.close();

        return list;

    }
}
