package cz.muni.fi.lesson;

import cz.muni.fi.exercise.DTDPage;
import cz.muni.fi.exercise.XPathPage;
import cz.muni.fi.exercise.XQueryPage;
import cz.muni.fi.exercise.XSDSchemaPage;
import cz.muni.fi.exercise.XSLPage;

/**
 * Class that represents Lesson
 *
 * @author Jiří Krejčí
 */
public class Lesson implements java.io.Serializable {

    private int ID;
    private String title;

    private enum LessonType {

        dtd, xsl, xpath, schema, xquery
    };
    
    private LessonType lessonType;
    private String text;

    public Lesson() {
    }

    public int getID() {
        return ID;
    }

    public void setID(int ID) {
        this.ID = ID;
    }

    public String getTitle() {
        return title;
    }

    public void setTitle(String title) {
        this.title = title;
    }

    public String getText() {
        return text;
    }

    public void setText(String text) {
        this.text = text;
    }

    public String getLessonType() {
        return lessonType.toString();
    }

    public void setLessonType(String lessonType) {

        if (lessonType.equals("dtd")) {
            this.lessonType = LessonType.dtd;
        } else if (lessonType.equals("xsl")) {
            this.lessonType = LessonType.xsl;
        } else if (lessonType.equals("xpath")) {
            this.lessonType = LessonType.xpath;
        } else if (lessonType.equals("schema")) {
            this.lessonType = LessonType.schema;
        } else if (lessonType.equals("xquery")) {
            this.lessonType = LessonType.xquery;
        } else {
            this.lessonType = null;
        }

    }

    /**
     * Method that returns class that specifes lesson type. 
     * Class is represented by Page with exercise.
     * 
     * @return Class for Page with exercise, if it exists, null otherwise.
     */
    public Class getLessonClass() {

        if (getLessonType().equals("dtd")) {
            return DTDPage.class;
        } else if (getLessonType().equals("xsl")) {
            return XSLPage.class;
        } else if (getLessonType().equals("xpath")) {
            return XPathPage.class;
        } else if (getLessonType().equals("schema")) {
            return XSDSchemaPage.class;
        } else if (getLessonType().equals("xquery")) {
            return XQueryPage.class;
        }

        return null;
    }
}
