package cz.muni.fi.lesson;

import cz.muni.fi.wicket.BasePage;
import org.apache.wicket.authroles.authorization.strategies.role.annotations.AuthorizeInstantiation;
import org.apache.wicket.markup.html.form.Button;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.model.ResourceModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.validation.IValidatable;
import org.apache.wicket.validation.validator.AbstractValidator;

/**
 * Class that reprezents page for editing lesson.
 * 
 * @author Jiří Krejčí
 */
@AuthorizeInstantiation("TEACHER")
public final class LessonEditPage extends BasePage implements java.io.Serializable {

    private final Lesson lesson;

    public LessonEditPage(PageParameters parameters) {
        super();

        LessonManager lessonManager = new LessonManager();
        lesson = lessonManager.getLesson(parameters.get("id").toOptionalInteger());

        add(new EditLessonForm("form", parameters));

        final FeedbackPanel feedback = new FeedbackPanel("feedback");
        feedback.setOutputMarkupId(true);
        add(feedback);

    }

    private class EditLessonForm extends Form {

        private TextArea lessonTextTextArea;
        private TextField lessonTextField;

        public EditLessonForm(String id, final PageParameters parameters) {
            super(id);
            
            lessonTextTextArea = new TextArea("lessonText", new PropertyModel(this, "lessonText") {
                @Override
                public Object getObject() {
                    return (String) lesson.getText();
                }

                @Override
                public void setObject(Object o) {
                    lesson.setText((String) o);
                }
            });
            lessonTextTextArea.add(new NicEditValidator());
            lessonTextTextArea.setLabel(new ResourceModel("form.lessonText"));
            add(lessonTextTextArea);

            lessonTextField = new TextField("lessonTitle", new PropertyModel(this, "lessonTitle") {
                @Override
                public Object getObject() {
                    return (String) lesson.getTitle();
                }

                @Override
                public void setObject(Object o) {
                    lesson.setTitle((String) o);
                }
            });
            lessonTextField.add(new NullValidator());
            lessonTextField.setLabel(new ResourceModel("form.lessonTitle"));
            add(lessonTextField);

            add(new Button("button") {
                @Override
                public void onSubmit() {

                    LessonManager lessonManager = new LessonManager();
                    lessonManager.updateLesson(lesson);

                    info("Obsah lekce byl úspěšně uložen.");
                    
                    setResponsePage(LessonPage.class, new PageParameters().set("type", parameters.get("responssPage").toString()));

                }
            });

        }

        /**
         * Class that validates null values in form.
         * Gets error on null value, on empty string or on string containg just "<br>".
         */
        private class NullValidator extends AbstractValidator {

            @Override
            public void onValidate(IValidatable iv) {

                if (iv.getValue() == null || ((String) iv.getValue()).equals("") || iv.getValue().equals("<br>")) {
                    error(iv);
                }

            }

            @Override
            public boolean validateOnNullValue() {
                return true;
            }

            @Override
            protected String resourceKey() {
                return "emptyArea";
            }
        }

        /**
         * Class that validates null values in form for NicEditor
         * Gets error on empty contet, that isrepresented by "<br>".
         */
        private class NicEditValidator extends AbstractValidator {

            @Override
            public void onValidate(IValidatable iv) {

                if (iv.getValue().equals("<br>")) {
                    error(iv);
                }

            }

            @Override
            public boolean validateOnNullValue() {
                return true;
            }

            @Override
            protected String resourceKey() {
                return "emptyArea";
            }
        }
    }
}
