package cz.muni.fi.lesson;

import cz.muni.fi.exercise.*;
import cz.muni.fi.wicket.AddExercisePage;
import cz.muni.fi.wicket.BasePage;
import cz.muni.fi.wicket.SignInSession;
import org.apache.wicket.MarkupContainer;
import org.apache.wicket.Session;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;

/**
 * Class that represents Page for Lesson
 *
 * @author Jiří Krejčí
 */
public final class LessonPage extends BasePage implements java.io.Serializable {

    private String title;
    private String text;

    public LessonPage(PageParameters parameters) {
        super();

        LessonManager lessonManager = new LessonManager();
        final Lesson lesson = lessonManager.getLesson(parameters.get("type").toOptionalString());
        title = lesson.getTitle();
        text = lesson.getText();

        add(new Label("title", new PropertyModel(this, "title")));
        add(new Label("text", new PropertyModel(this, "text")).setEscapeModelStrings(false));

        add(new BookmarkablePageLink("edit", LessonEditPage.class, new PageParameters().add("id", lesson.getID()).add("responssPage", lesson.getLessonType())) {
            @Override
            public boolean isVisible() {
                SignInSession session = (SignInSession) Session.get();
                if (session == null || session.getUser() == null) {
                    return false;
                }
                return session.getUser().getRoles().toString().equals("TEACHER");
            }
        });

        PropertyModel hibernateExerciseListModel = new PropertyModel(this, "exerciseList") {
            @Override
            public Object getObject() {
                ExerciseManager manager = new ExerciseManager();
                return manager.listAllExercises(lesson.getLessonType());
            }
        };
        add(new ListView("exerciseList", hibernateExerciseListModel) {
            protected void populateItem(ListItem item2) {

                Exercise e = (Exercise) item2.getModelObject();

                BookmarkablePageLink link;
                link = new BookmarkablePageLink("link", lesson.getLessonClass(), new PageParameters().set("id", e.getID()));
                link.add(new Label("exercise", "Cvičení " + String.valueOf(item2.getIndex() + 1) + "."));
                item2.add(link);

            }
        });

        add(new MarkupContainer("addContainer") {
            @Override
            public boolean isVisible() {
                SignInSession session = (SignInSession) Session.get();
                if (session == null || session.getUser() == null) {
                    return false;
                }
                return session.getUser().isTeacher();
            }
        }.add(new BookmarkablePageLink("add", AddExercisePage.class)));
    }
}
