package cz.muni.fi.processor;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.StringReader;
import java.io.StringWriter;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import org.apache.xerces.parsers.DOMParser;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXNotRecognizedException;
import org.xml.sax.SAXNotSupportedException;
import org.xml.sax.SAXParseException;

/**
 * Class that handles DTD validation.
 *
 * @author Jiří Krejčí
 */
public class DTDProcessor extends AProcessor implements java.io.Serializable {

    private File tempDTDFile;

    public DTDProcessor(String xml, String dtd, String outputXml) {
        super(xml, dtd, outputXml);
        tempDTDFile = new File("tempDTD.dtd");
    }

    public File getDTD() {
        return tempDTDFile;
    }

    public void setDTD(String param) throws IOException {

        if (param == null) {
            param = "";
        }

        BufferedWriter bf = new BufferedWriter(new FileWriter(tempDTDFile));
        bf.write(param);
        bf.close();

    }

    @Override
    public void validate() {

        clearResultMessage();
        clearResultXML();

        try {

            setDTD(getInputXML());

            TransformerFactory transformerFactory = TransformerFactory.newInstance();
            Transformer transformer = transformerFactory.newTransformer();

            Source source = new StreamSource(new StringReader((getTaskXml())));
            StreamResult streamResult = new StreamResult(new StringWriter());

            transformer.setParameter(OutputKeys.INDENT, "yes");
            transformer.setOutputProperty(OutputKeys.DOCTYPE_SYSTEM, tempDTDFile.getAbsolutePath());

            transformer.transform(source, streamResult);

            DOMParser parser = new DOMParser();
            parser.setErrorHandler(new ErrorHandler(this));

            parser.setDTDSource(null);
            parser.setFeature("http://xml.org/sax/features/validation", true);

            parser.parse(new InputSource(new StringReader(streamResult.getWriter().toString())));

        } catch (SAXNotRecognizedException ex) {
            Logger.getLogger(DTDProcessor.class.getName()).log(Level.SEVERE, null, ex);
        } catch (SAXNotSupportedException ex) {
            Logger.getLogger(DTDProcessor.class.getName()).log(Level.SEVERE, null, ex);
        } catch (SAXParseException ex) {
            Logger.getLogger(DTDProcessor.class.getName()).log(Level.SEVERE, null, ex);
        } catch (SAXException ex) {
            Logger.getLogger(DTDProcessor.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(DTDProcessor.class.getName()).log(Level.SEVERE, null, ex);
        } catch (Exception ex) {
            Logger.getLogger(DTDProcessor.class.getName()).log(Level.SEVERE, null, ex);
        }

    }

    @Override
    public boolean isValidationCorrect() {
        return (getResultMessage() == null || getResultMessage().equals(""));
    }

    @Override
    public String getInfoMessage() {
        return "DTD je správně.";
    }

    @Override
    public String getErrorMessage() {
        return getResultMessage();
    }
}
