package cz.muni.fi.processor;

import java.io.StringReader;
import java.io.StringWriter;
import java.io.Writer;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathFactory;
import org.apache.xerces.parsers.DOMParser;
import org.w3c.dom.*;
import org.xml.sax.InputSource;

/**
 * Class that handles xPath query validation.
 *
 * @author Jiří Krejčí
 */
public class XPathProcessor extends AProcessor implements java.io.Serializable {

    private Document document;

    public XPathProcessor(String xml, String xpath, String outputXml) {
        super(xml, xpath, outputXml);
    }

    /**
     * Method taht displays the result in the form of a well-formed XML
     * document.
     *
     */
    private String show() {

        try {

            DOMParser parser = new DOMParser();

            parser.parse(new InputSource(new StringReader(getTaskXml())));
            document = parser.getDocument();

            XPathFactory xPathFactory = XPathFactory.newInstance();
            XPath xPath = xPathFactory.newXPath();

            if (getInputXML().equals("") || getInputXML() == null) {
                setInputXML("/");
            }
            NodeList nodeList = (NodeList) xPath.evaluate(getInputXML(),
                    document.getDocumentElement(),
                    XPathConstants.NODESET); //NODESET je sezam uzlu


            Element rootElement = document.createElement("doc");

            for (int i = 0; i < nodeList.getLength(); i++) {

                //If the item is Element, add element
                if (nodeList.item(i) instanceof Element) {
                    rootElement.appendChild(nodeList.item(i));
                } //If the item is Attribute, add attribute
                else if (nodeList.item(i) instanceof Attr) {
                    Attr attribute = (Attr) nodeList.item(i);
                    rootElement.setAttribute(attribute.getName(), attribute.getTextContent());

                } //Othewise add Document
                else {
                    Document pomDoc = (Document) nodeList.item(i);
                    rootElement.appendChild(pomDoc.getDocumentElement());
                }
            }

            TransformerFactory transformerFactory = TransformerFactory.newInstance();
            Transformer trace = transformerFactory.newTransformer();

            Writer writer = new StringWriter();

            Source source = new DOMSource(rootElement);
            
            //StreamResult sr = new StreamResult(System.out);
            StreamResult streamResult = new StreamResult(writer);

            trace.setErrorListener(new ErrorListener(this));

            trace.setOutputProperty(OutputKeys.INDENT, "yes");
            trace.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "4");

            trace.transform(source, streamResult);
            //trace.transform(doms, sr);

            return streamResult.getWriter().toString();

        } catch (Exception ex) {
            Logger.getLogger(XPathProcessor.class.getName()).log(Level.SEVERE, null, ex);
        }

        return "";

    }

    @Override
    public void validate() {

        clearResultMessage();
        clearResultXML();

        try {

            setResultXML(show());

        } catch (Exception ex) {
            Logger.getLogger(XPathProcessor.class.getName()).log(Level.SEVERE, null, ex);
        }


    }

    @Override
    public boolean isValidationCorrect() {
        return (getResultXML().replaceAll("\\s", "").equals(getOutputXML().replaceAll("\\s", "")));
    }

    @Override
    public String getInfoMessage() {
        return "Váš výsledek je správně.";
    }

    @Override
    public String getErrorMessage() {
        return "Váš výsledek neodpovídá očekávanému výsledku. " + getResultMessage();
    }
}