package cz.muni.fi.processor;

import java.io.StringReader;
import java.io.StringWriter;
import java.util.Properties;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.sax.SAXSource;
import javax.xml.transform.stream.StreamResult;
import net.sf.saxon.Configuration;
import net.sf.saxon.om.DocumentInfo;
import net.sf.saxon.query.DynamicQueryContext;
import net.sf.saxon.query.StaticQueryContext;
import net.sf.saxon.query.XQueryExpression;
import net.sf.saxon.trans.XPathException;
import org.xml.sax.InputSource;

/**
 * Class that handles xQuery validation.
 *
 * @author Franta
 */
public class XQueryProcessor extends AProcessor implements java.io.Serializable {

    public XQueryProcessor(String xml, String xQueryExpression, String outXml) {
        super(xml, xQueryExpression, outXml);
    }

    @Override
    public void validate() {

        clearResultMessage();
        clearResultXML();

        try {

            Configuration c = new Configuration();

            StaticQueryContext sqc = new StaticQueryContext(c);
            DynamicQueryContext dqc = new DynamicQueryContext(c);

            sqc.setErrorListener(new ErrorListener(this));

            final XQueryExpression exp = sqc.compileQuery(getInputXML());

            InputSource XMLSource = new InputSource(new StringReader(getTaskXml()));
            SAXSource SAXs = new SAXSource(XMLSource);
            DocumentInfo DI = sqc.buildDocument(SAXs);
            dqc.setContextNode(DI);

            Properties properties = new Properties();
            properties.setProperty(OutputKeys.METHOD, "xml");
            properties.setProperty(OutputKeys.INDENT, "yes");

            StringWriter stringWriter = new StringWriter();

            exp.run(dqc, new StreamResult(stringWriter), properties);

            setResultXML(stringWriter.toString());

        } catch (XPathException ex) {
            setResultMessage(getResultMessage() + ex.toString());
        }

    }

    @Override
    public boolean isValidationCorrect() {
        return (getResultXML().replaceAll("\\s", "").equals(getOutputXML().replaceAll("\\s", "")));
    }

    @Override
    public String getInfoMessage() {
        return "Váš výsledek je správně.";
    }

    @Override
    public String getErrorMessage() {
        return "Váš výsledek neodpovídá očekávanému výsledku. " + getResultMessage();
    }
}
