package cz.muni.fi.processor;

import java.io.IOException;
import java.io.StringReader;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;
import org.xml.sax.SAXException;

/**
 * Class that handles XSD Schema validation.
 *
 * @author Jiří Krjčí
 */
public class XSDSchemaProcessor extends AProcessor {

    public XSDSchemaProcessor(String xml, String xsd, String outputXml) {
        super(xml, xsd, outputXml);
    }

    @Override
    public void validate() {

        clearResultMessage();
        clearResultXML();

        try {

            String schemaLang = "http://www.w3.org/2001/XMLSchema";

            SchemaFactory factory = SchemaFactory.newInstance(schemaLang);

            Schema schema = factory.newSchema(new StreamSource(new StringReader(getInputXML())));
            Validator validator = schema.newValidator();

            validator.setErrorHandler(new ErrorHandler(this));

            validator.validate(new StreamSource(new StringReader(getTaskXml())));


        } catch (SAXException ex) {
            setResultMessage(getResultMessage() + "Exception: " + ex.getMessage().toString());
        } catch (IOException ex) {
            Logger.getLogger(XSDSchemaProcessor.class.getName()).log(Level.SEVERE, null, ex);
        }

    }

    @Override
    public boolean isValidationCorrect() {
        return (getResultMessage() == null || getResultMessage().equals(""));
    }

    @Override
    public String getInfoMessage() {
        return "XSD schéma je správně.";
    }

    @Override
    public String getErrorMessage() {
        return "Chyba: <br />" + getResultMessage();
    }
}
