package cz.muni.fi.processor;

import java.io.StringReader;
import java.io.StringWriter;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

/**
 * Class that handles XSL transformation validation.
 *
 * @author Jiří Krejčí
 */
public class XSLProcessor extends AProcessor implements java.io.Serializable {

    public XSLProcessor(String xml, String xsl, String outXml) {
        super(xml, xsl, outXml);
    }

    @Override
    public void validate() {

        clearResultMessage();
        clearResultXML();

        ErrorListener el = new ErrorListener(this);

        try {

            TransformerFactory factory = TransformerFactory.newInstance();
            factory.setErrorListener(el);

            StreamSource xslStream = new StreamSource(new StringReader(getInputXML()));

            Transformer transformer = factory.newTransformer(xslStream);
            transformer.setErrorListener(el);

            StreamSource in = new StreamSource(new StringReader(getTaskXml()));
            StreamResult out = new StreamResult(new StringWriter());

            transformer.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "4");
            transformer.setOutputProperty(OutputKeys.INDENT, "yes");

            transformer.transform(in, out);
//            transformer.transform(xslStream, out);

            setResultXML(out.getWriter().toString());

        } catch (TransformerConfigurationException ex) {
            Logger.getLogger(XSLProcessor.class.getName()).log(Level.SEVERE, null, ex);
        } catch (TransformerException ex) {
            Logger.getLogger(XSLProcessor.class.getName()).log(Level.SEVERE, null, ex);
        }

    }

    @Override
    public boolean isValidationCorrect() {
        return (getResultXML().replaceAll("\\s", "").equals(getOutputXML().replaceAll("\\s", "")));
    }

    @Override
    public String getInfoMessage() {
        return "Transformace se shoduje s očekáváným výsledekm.";
    }

    @Override
    public String getErrorMessage() {
        return "Chyba: " + getResultMessage();
    }
}
