package cz.muni.fi.user;

import cz.muni.fi.wicket.BasePage;
import org.apache.wicket.feedback.ContainerFeedbackMessageFilter;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.PasswordTextField;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.model.Model;
import org.apache.wicket.validation.validator.PatternValidator;
import org.apache.wicket.validation.validator.StringValidator;

/**
 * Class that represents Page for Registration.
 *
 * @author Jiří Krejčí
 */
public final class RegistrationPage extends BasePage {

    public RegistrationPage() {
        super();

        AddUserForm form = new AddUserForm("registrationForm");
        add(form);

        FeedbackPanel feedback = new FeedbackPanel("registrationFeedback", new ContainerFeedbackMessageFilter(form));
        add(feedback);

    }

    private class AddUserForm extends Form {

        private TextField username;
        private TextField uco;
        private TextField firstName;
        private TextField lastName;
        private PasswordTextField password;
        private PasswordTextField registrationPassword;

        public AddUserForm(String id) {
            super(id);

            username = new TextField("username", new Model());
            username.setRequired(true);
            username.add(StringValidator.lengthBetween(5, 20));
            add(username);

            uco = new TextField("uco", new Model());
            uco.setRequired(true);
            uco.add(new PatternValidator("\\d+"));
            add(uco);

            firstName = new TextField("firstName", new Model());
            firstName.setRequired(true);
            add(firstName);

            lastName = new TextField("lastName", new Model());
            lastName.setRequired(true);
            add(lastName);

            password = new PasswordTextField("pw", new Model());
            password.setRequired(true);
            password.add(StringValidator.lengthBetween(5, 20));
            add(password);

            registrationPassword = new PasswordTextField("regPw", new Model());
            registrationPassword.setRequired(true);
            add(registrationPassword);

        }

        @Override
        protected void onSubmit() {

            UserManager userManager = new UserManager();

            if (userManager.isRegistrationAlowed((String) registrationPassword.getModelObject())) {

                if (userManager.addUser((String) username.getModelObject(),
                        (String) uco.getModelObject(),
                        (String) firstName.getModelObject(),
                        (String) lastName.getModelObject(),
                        (String) password.getModelObject(),
                        (String) registrationPassword.getModelObject())) {

                    clearForm();
                    clearInput();

                    info("Uživatel úspěšně zaregistrován.");

                } else {

                    error("Uživatelské jméno je již zaregistrováno.");
                }


            } else {

                error("Registrace není v tomto období dovolena nebo neznáte správné heslo.");

            }
        }

        private void clearForm() {
            username.setModelObject(null);
            uco.setModelObject(null);
            firstName.setModelObject(null);
            lastName.setModelObject(null);
        }
    }
}
