package cz.muni.fi.user;

import cz.muni.fi.hibernate.Database;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Manager that controls Registration Period
 *
 * @author Jiří Krejčí
 */
public class RegistrationPeriodManager implements java.io.Serializable {

    Database d;

    public RegistrationPeriodManager() {
        d = new Database();
    }

    /**
     * Method that adds new Registraion Period
     *
     * @param from
     * @param to
     * @param password
     * @param role
     * @return True if period succesfully added, false otherwise
     */
    public boolean addRegistrationPeriod(String from, String to, String password, SelectOption role) {


        if (d.getRegistration(password) != null) {

            return false;

        } else {


            RegistrationPeriod rp = new RegistrationPeriod();

            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            try {
                rp.setFrom(sdf.parse(from + ":00"));
                rp.setTo(sdf.parse(to + ":00"));
            } catch (ParseException ex) {
                Logger.getLogger(RegistrationPeriodPage.class.getName()).log(Level.SEVERE, null, ex);
            }

            rp.setPassword(password);

            if (role == null) {
                rp.setRole("STUDENT");
            } else {
                rp.setRole(role.getKey());
            }

            d.addRegistrationPeriod(rp);

            TagManager tagManager = new TagManager();
            tagManager.addTag(password);

            return true;
        }

    }

    /**
     * Method that lists all registration periods.
     *
     * @return List of all registration periods.
     */
    public List<RegistrationPeriod> listAllRegistrationPeriods() {

        return d.listAllRegistrationPeriod();

    }

    /**
     * Method that gets registraion period with specifed password
     *
     * @param password Pasword to be searched
     * @return Period with specifed password
     */
    public RegistrationPeriod getRegistrationPeriod(String password) {

        return d.getRegistration(password);

    }

    /**
     * Method that says if the registration is allowd
     *
     * @param password Registratino period passwod
     * @return True if the registration is allowed, false othewise
     */
    public boolean isRegistrationAlowed(String password) {

        RegistrationPeriod rp = getRegistrationPeriod(password);

        if (rp.getPassword() == null) {

            return false;

        } else {

            return true;

        }

    }
}
