package cz.muni.fi.user;

import cz.muni.fi.wicket.BasePage;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import org.apache.wicket.authroles.authorization.strategies.role.annotations.AuthorizeInstantiation;
import org.apache.wicket.feedback.ContainerFeedbackMessageFilter;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.ChoiceRenderer;
import org.apache.wicket.markup.html.form.DropDownChoice;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.PasswordTextField;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.model.Model;
import org.apache.wicket.validation.validator.StringValidator;

/**
 * Class that represent Page for Registration Period.
 *
 * @author Franta
 */
@AuthorizeInstantiation("TEACHER")
public final class RegistrationPeriodPage extends BasePage implements java.io.Serializable {

    public RegistrationPeriodPage() {
        super();

        RegistrationPeriodManager rpm = new RegistrationPeriodManager();

        add(new ListView("listAllPeriods", rpm.listAllRegistrationPeriods()) {
            @Override
            protected void populateItem(ListItem item) {
                RegistrationPeriod p = (RegistrationPeriod) item.getModelObject();
                item.add(new Label("index", String.valueOf(item.getIndex() + 1) + "."));
                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                item.add(new Label("from", sdf.format(p.getFrom())));
                item.add(new Label("to", sdf.format(p.getTo())));
                item.add(new Label("password", p.getPassword()));
                item.add(new Label("role", p.getRole()));
            }
        });


        AddRegistrationPeriodForm form = new AddRegistrationPeriodForm("registrationPeriodForm");
        add(form);

        FeedbackPanel feedback = new FeedbackPanel("registrationPeriodFeedback", new ContainerFeedbackMessageFilter(form));
        add(feedback);

    }

    private class AddRegistrationPeriodForm extends Form {

        private TextField from;
        private TextField to;
        private PasswordTextField password;
        private DropDownChoice listRoles;

        public AddRegistrationPeriodForm(String id) {
            super(id);

            from = new TextField("from", new Model());
            from.setRequired(true);
            add(from);

            to = new TextField("to", new Model());
            to.setRequired(true);
            add(to);

            password = new PasswordTextField("pw", new Model());
            password.setRequired(true);
            password.add(StringValidator.lengthBetween(5, 20));
            add(password);


            List<SelectOption> rolesTypeList = new ArrayList<SelectOption>();

            rolesTypeList.add(new SelectOption("STUDENT", "Student"));
            rolesTypeList.add(new SelectOption("TEACHER", "Učitel"));

            ChoiceRenderer choiceRenderer = new ChoiceRenderer("value", "key");
            listRoles = new DropDownChoice("rolesList", new Model(), rolesTypeList, choiceRenderer);
            add(listRoles);
        }

        @Override
        protected void onSubmit() {

            RegistrationPeriodManager rpm = new RegistrationPeriodManager();

            if (rpm.addRegistrationPeriod(
                    ((String) from.getModelObject() + ":00"),
                    ((String) to.getModelObject() + ":00"),
                    (String) password.getModelObject(),
                    (SelectOption) listRoles.getModelObject())) {

                clearForm();
                clearInput();

                info("Registrační období úspěšně uloženo.");

            } else {

                error("Zvolené registrační heslo je již používáno, zvolte prosím heslo jíné.");

            };

        }

        private void clearForm() {
            from.setModelObject(null);
            to.setModelObject(null);
            listRoles.setModelObject(null);
        }
    }
}
