package cz.muni.fi.user;

import cz.muni.fi.exercise.Exercise;
import cz.muni.fi.exercise.ExerciseManager;
import cz.muni.fi.exercise.UserExerciseProtocol;
import cz.muni.fi.wicket.BasePage;
import cz.muni.fi.wicket.SignInSession;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import org.apache.wicket.Session;
import org.apache.wicket.behavior.AttributeAppender;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.model.Model;
import org.apache.wicket.request.mapper.parameter.PageParameters;

/**
 * Class that represents Page for List od UserExerices
 * 
 * @author Jiří Krejčí
 */
public final class UserExercicesListPage extends BasePage implements java.io.Serializable {

    public UserExercicesListPage() {
        super();

        List<String> types = new ArrayList<String>();
        types.add("DTD");
        types.add("XSL");
        types.add("xPath");
        types.add("Schema");
        types.add("xQuery");

        add(new ListView("exercisesList", types) {
            @Override
            protected void populateItem(ListItem exerciseType) {

                exerciseType.add(new Label("exerciceType", ((String) exerciseType.getModelObject()).toLowerCase()));

                ExerciseManager exerciseManager = new ExerciseManager();

                exerciseType.add(new ListView("exercise", exerciseManager.listAllExercises((String) exerciseType.getModelObject())) {
                    @Override
                    protected void populateItem(ListItem exercise) {

                        Exercise ex = (Exercise) exercise.getModelObject();

                        UserManager um = new UserManager();

                        SignInSession session = (SignInSession) Session.get();
                        final List userExerciseList = um.getUserExerciceProtocols(session.getUser(), ex);

                        exercise.add(new Label("protocolNotDone", "NOK") {
                            @Override
                            public boolean isVisible() {
                                return (userExerciseList.isEmpty());
                            }
                        }.add(new AttributeAppender("class", new Model<String>(" no"), " ")));

                        exercise.add(new ListView("protocol", userExerciseList) {
                            @Override
                            protected void populateItem(ListItem protocol) {
                                UserExerciseProtocol userExercise = (UserExerciseProtocol) protocol.getModelObject();

                                //System.out.println(exInfo);
                                PageParameters params = new PageParameters();
                                params.add("userExerciseId", userExercise.getID());
                                BookmarkablePageLink linkToProtocol = new BookmarkablePageLink("protocolInfo", UserExerciseProtocolPage.class, params);

                                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                                linkToProtocol.add(new AttributeAppender("title", new Model<String>("Uloženo: " + sdf.format(userExercise.getTimestamp()))));

                                if (userExercise.isSuccesulyPassed()) {
                                    linkToProtocol.add(new Label("protocolInfoContent", "OK "));
                                    linkToProtocol.add(new AttributeAppender("class", new Model<String>(" yes"), " "));
                                } else {
                                    linkToProtocol.add(new Label("protocolInfoContent", "NOK "));
                                    linkToProtocol.add(new AttributeAppender("class", new Model<String>(" no"), " "));
                                }

                                protocol.add(linkToProtocol);
                            }
                        });
                    }
                });

            }
        });

    }
}
