package cz.muni.fi.user;

import cz.muni.fi.exercise.Exercise;
import cz.muni.fi.exercise.ExerciseManager;
import cz.muni.fi.exercise.UserExerciseProtocol;
import cz.muni.fi.exercise.UserExerciseProtocolManager;
import cz.muni.fi.wicket.BasePage;
import cz.muni.fi.wicket.SignInSession;
import java.text.SimpleDateFormat;
import org.apache.wicket.extensions.ajax.markup.html.AjaxEditableLabel;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.model.CompoundPropertyModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;

/**
 * Class that represents Protocols.
 * 
 * @author Jiří Krejčí
 */
public final class UserExerciseProtocolPage extends BasePage implements java.io.Serializable {
    
    private String comment;
    private UserExerciseProtocol protocol;

    public UserExerciseProtocolPage(PageParameters params) {
        super();
        setDefaultModel(new CompoundPropertyModel<UserExerciseProtocolPage>(this));
       
        UserExerciseProtocolManager exerciseProtocolManager = new UserExerciseProtocolManager();
        protocol = exerciseProtocolManager.getProtocol(params.get("userExerciseId").toInt());
        
        UserManager userManager = new UserManager();
        User user = userManager.getUser(protocol.getUserExerciseId().getUser_id());
        add(new Label("user", user.getFirstname() + " " + user.getLastname() + " (" + user.getUsername() + ")"));
        
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        add(new Label("timestamp", sdf.format(protocol.getTimestamp())));
        
        ExerciseManager exerciseManager = new ExerciseManager();
        
        Exercise e = exerciseManager.getExercise(protocol.getUserExerciseId().getExercise_id());
        add(new Label("exercise", e.getTask()));
        add(new Label("taskXML", e.getTaskXML()));
        add(new Label("resultXML", protocol.getResult()));
        SignInSession session = (SignInSession) getSession();
        if (session.getUser().isTeacher()) {
            add(new AjaxEditableLabel("comment"));
        } else {
            add(new Label("comment"));
        }
        
    }

    public String getComment() {
        return protocol.getComment();
    }

    public void setComment(String comment) {
        
        protocol.setComment(comment);
        
        UserExerciseProtocolManager exerciseProtocolManager = new UserExerciseProtocolManager();
        exerciseProtocolManager.updateProtocol(protocol);
       
    }
    
    
}
