package cz.muni.fi.user;

import cz.muni.fi.exercise.Exercise;
import cz.muni.fi.exercise.UserExerciseProtocol;
import cz.muni.fi.exercise.UserExerciseProtocolManager;
import cz.muni.fi.hibernate.Database;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import org.apache.wicket.authroles.authorization.strategies.role.Roles;

/**
 * Manager that controls User.
 * 
 * @author Jiří Krejčí
 */
public class UserManager implements java.io.Serializable {
    
    Database d;

    public UserManager() {
        
        d = new Database();
        
    }

    public User getUser (String username) {

        final User user = d.getUser(username);

        if (user.getUsername() == null || user.getPassword() == null || user.getFirstname() == null || user.getLastname() == null || user.getRole() == null) {
            return null;
        }

        return user;
    }
    
    public User getUser(Integer userId) {
    
        return d.getUser(userId);
        
    }

    public boolean addUser(String username, String uco, String firstName, String lastName, String password, String registrationPassword) {

        User user = new User();

        user.setUco(Long.valueOf(uco));
        user.setUsername(username.toLowerCase());
        user.setFirstname(firstName);
        user.setLastname(lastName);
        user.setPassword(password);

        if (d.getUser(username).getUsername() != null) {

            return false;

        } else {

            RegistrationPeriodManager rpm = new RegistrationPeriodManager();

            Roles r = new Roles(rpm.getRegistrationPeriod(registrationPassword).getRole());
            user.setRole(r.toString());

            d.addUser(user);

            addUserTag(user, registrationPassword);

            return true;

        }
    }

    private void addUserTag(User user, String tagName) {
        
        TagManager tm = new TagManager();

        tm.addUserTag(user, tm.getTag(tagName));

    }

    public boolean isRegistrationAlowed(String password) {
        
        RegistrationPeriodManager rpm = new RegistrationPeriodManager();
        
        return rpm.isRegistrationAlowed(password);

    }

    public List<User> getAllUsers() {

        return d.listAllUsers();

    }

    public List<User> getAllUsers(Tag t, Roles r) {
        
        List<User> list = new ArrayList<User>();

        Iterator iter = r.iterator();
        while (iter.hasNext()) {
            list.addAll(d.listAllUsers(t, new Roles((String) iter.next())));
        }
        return list;

    }

    public List<User> getAllUsers(Tag t) {
        
        return d.listAllUsers(t);

    }

    public List<User> getAllUsers(Roles r) {

        List<User> list = new ArrayList<User>();

        Iterator iter = r.iterator();
        while (iter.hasNext()) {
            list.addAll(d.listAllUsers(new Roles((String) iter.next())));
        }
        return list;
    }

    public List<Tag> getUserTags(User user) {

        return d.listUserTags(user);

    }

    public List<UserExerciseProtocol> getUserExerciceProtocols(User user, Exercise exercise) {
        
        UserExerciseProtocolManager exerciseProtocolManager = new UserExerciseProtocolManager();
        
        return exerciseProtocolManager.listProtocols(user, exercise);

    }
}
