package cz.muni.fi.user;

import cz.muni.fi.exercise.Exercise;
import cz.muni.fi.exercise.ExerciseManager;
import cz.muni.fi.exercise.UserExerciseProtocol;
import cz.muni.fi.wicket.BasePage;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import org.apache.wicket.MarkupContainer;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.form.AjaxFormComponentUpdatingBehavior;
import org.apache.wicket.authroles.authorization.strategies.role.Roles;
import org.apache.wicket.behavior.AttributeAppender;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.ChoiceRenderer;
import org.apache.wicket.markup.html.form.DropDownChoice;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.request.mapper.parameter.PageParameters;

/**
 * Class that represents Page for Users List
 * @author Franta
 */
public final class UseresListPage extends BasePage implements java.io.Serializable {
    
    private Tag filterTag;
    private Roles filterRoles;
    
    public UseresListPage() {
        super();
        
        Form form = new Form("form");

        final DropDownChoice tagDropDownChoise = new DropDownChoice("listOfTags",
                new Model(),
                new TagManager().listAllTags(),
                new ChoiceRenderer("name"));
        form.add(tagDropDownChoise);
        
        List<String> rolesList = new ArrayList<String>();
        rolesList.add("STUDENT");
        rolesList.add("TEACHER");
        
        final DropDownChoice roleDropDownChoise = new DropDownChoice("listOfRoles", new Model(), rolesList);
        form.add(roleDropDownChoise);
        
        final UserManager um = new UserManager();
        
        LoadableDetachableModel loadableDetachableModel = new LoadableDetachableModel() {
            @Override
            protected Object load() {
                
                if (filterTag != null && filterRoles != null) {
                    System.out.println("(filterTag != null && filterRoles != null)");
                    return um.getAllUsers(filterTag, filterRoles);
                } else if (filterRoles == null && filterTag != null) {
                    System.out.println("(filterRoles == null && filterTag != null)");
                    return um.getAllUsers(filterTag);
                } else if (filterTag == null && filterRoles != null) {
                    System.out.println("(filterTag == null && filterRoles != null)");
                    return um.getAllUsers(filterRoles);
                } else {
                    System.out.println("ELSE");
                    return um.getAllUsers();
                }
                
            }
        };
        
        final ListView view = new ListView("listAllUsers", loadableDetachableModel) {
            @Override
            protected void populateItem(ListItem item) {
                final User user = (User) item.getModelObject();
                item.add(new Label("numeration", String.valueOf(item.getIndex() + 1) + "."));
//                item.add(new Label("id", String.valueOf(user.getID())));
                item.add(new Label("username", user.getUsername()));
                item.add(new Label("uco", String.valueOf(user.getUco())));
                item.add(new Label("firstName", user.getFirstname()));
                item.add(new Label("lastName", user.getLastname()));
//                item.add(new Label("role", user.getRole()));

//                item.add(new ListView("tags", um.getUserTags(user)) {
//                    @Override
//                    protected void populateItem(ListItem item) {
//
//                        Tag t = (Tag) item.getModelObject();
//                        item.add(new Label("tag", t.getName()));
//
//                    }
//                });

                List<String> types = new ArrayList<String>();
                types.add("dtd");
                types.add("xsl");
                types.add("xpath");
                types.add("schema");
                types.add("xquery");
                
                item.add(new ListView("exerciseType", types) {
                    @Override
                    protected void populateItem(ListItem exercises) {
                        
                        ExerciseManager exerciseManager = new ExerciseManager();
                        exercises.add(new ListView("exercises", exerciseManager.listAllExercises((String) exercises.getModelObject())) {
                            @Override
                            protected void populateItem(ListItem exercisesType) {
                                
                                exercisesType.add(new Label("exerciseNumber", String.valueOf(exercisesType.getIndex() + 1) + ". "));
                                
                                final Exercise exercise = (Exercise) exercisesType.getModelObject();
                                final List userExerciseList = um.getUserExerciceProtocols(user, exercise);
                                
                                exercisesType.add(new Label("exerciseNotDone", "NOK") {
                                    @Override
                                    public boolean isVisible() {
                                        return (userExerciseList.isEmpty());
                                    }
                                }.add(new AttributeAppender("class", new Model<String>(" no"), " ")));
                                
                                
                                exercisesType.add(new ListView("exercise", userExerciseList) {
                                    @Override
                                    protected void populateItem(ListItem protocol) {
                                        UserExerciseProtocol userExercise = (UserExerciseProtocol) protocol.getModelObject();

                                        //System.out.println(exInfo);
                                        PageParameters params = new PageParameters();
                                        params.add("userExerciseId", userExercise.getID());
                                        BookmarkablePageLink linkToProtocol = new BookmarkablePageLink("exerciseInfo", UserExerciseProtocolPage.class, params);
                                        
                                        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                                        linkToProtocol.add(new AttributeAppender("title", new Model<String>("Uloženo: " + sdf.format(userExercise.getTimestamp()))));
                                        
                                        if (userExercise.isSuccesulyPassed()) {
                                            linkToProtocol.add(new Label("exerciseInfoContent", "OK "));
                                            linkToProtocol.add(new AttributeAppender("class", new Model<String>(" yes"), " "));
                                        } else {
                                            linkToProtocol.add(new Label("exerciseInfoContent", "NOK "));
                                            linkToProtocol.add(new AttributeAppender("class", new Model<String>(" no"), " "));
                                        }
                                        
                                        protocol.add(linkToProtocol);
                                    }
                                });
                                
                                
                                
                            }
                        });
                        
                    }
                });
                
            }
        };
        
        final MarkupContainer tableContainer = new MarkupContainer("tableContainer") {
        };
        tableContainer.add(view);
        tableContainer.setOutputMarkupId(true);
        
        tagDropDownChoise.add(new AjaxFormComponentUpdatingBehavior("onchange") {
            @Override
            protected void onUpdate(AjaxRequestTarget target) {
                filterTag = (Tag) tagDropDownChoise.getModelObject();
                if (((String) roleDropDownChoise.getModelObject() == null)) {
                    filterRoles = new Roles("STUDENT, TEACHER");
                } else {
                    filterRoles = new Roles((String) roleDropDownChoise.getModelObject());
                }
                target.add(tableContainer);
            }
        });
        roleDropDownChoise.add(new AjaxFormComponentUpdatingBehavior("onchange") {
            @Override
            protected void onUpdate(AjaxRequestTarget target) {
                filterTag = (Tag) tagDropDownChoise.getModelObject();
                if (((String) roleDropDownChoise.getModelObject() == null)) {
                    filterRoles = new Roles("STUDENT, TEACHER");
                } else {
                    filterRoles = new Roles((String) roleDropDownChoise.getModelObject());
                }
                target.add(tableContainer);
            }
        });
        
        form.add(tableContainer);
        add(form);
        
        
    }
}
