package cz.muni.fi.wicket;

import cz.muni.fi.exercise.ExerciseManager;
import java.util.ArrayList;
import java.util.List;
import org.apache.wicket.authroles.authorization.strategies.role.annotations.AuthorizeInstantiation;
import org.apache.wicket.markup.html.form.Button;
import org.apache.wicket.markup.html.form.ChoiceRenderer;
import org.apache.wicket.markup.html.form.DropDownChoice;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.ResourceModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.validation.IValidatable;
import org.apache.wicket.validation.validator.AbstractValidator;

/**
 * Class that represents Page for Add exercise
 *
 * @author Jiří Krejčí
 */
@AuthorizeInstantiation("TEACHER")
public final class AddExercisePage extends BasePage implements java.io.Serializable {

    public AddExercisePage(PageParameters params) {
        super();

        AddExerciseForm form = new AddExerciseForm("form");
        add(form);

        final FeedbackPanel feedback = new FeedbackPanel("feedback");
        feedback.setOutputMarkupId(true);
        add(feedback);
    }

    private class AddExerciseForm extends Form {

        TextArea task;
        TextArea taskXML;
        TextArea inputXML;
        TextArea expectedResult;
        TextArea expectedResultXML;
        TextArea outputXML;
        DropDownChoice exerciseType;
        DropDownChoice showExpectedResult;

        public AddExerciseForm(String id) {
            super(id);

            task = new TextArea("task", new Model());
            task.add(new NullValidator());
            task.add(new NiceEditorVlaidator());
            task.setLabel(new ResourceModel("form.task"));
            add(task);

            taskXML = new TextArea("taskXML", new Model());
            taskXML.add(new NullValidator());
            taskXML.setLabel(new ResourceModel("form.taskXML"));
            add(taskXML);

            inputXML = new TextArea("inputXML", new Model());
            inputXML.add(new NullValidator());
            inputXML.setLabel(new ResourceModel("form.inputXML"));
            add(inputXML);

            expectedResult = new TextArea("expectedResult", new Model());
            expectedResult.add(new NullValidator());
            expectedResult.add(new NiceEditorVlaidator());
            expectedResult.setLabel(new ResourceModel("form.expectedResult"));
            add(expectedResult);

            expectedResultXML = new TextArea("expectedResultXML", new Model());
            expectedResultXML.add(new NullValidator());
            expectedResultXML.setLabel(new ResourceModel("form.expectedResultXML"));
            add(expectedResultXML);

            outputXML = new TextArea("outputXML", new Model());

            add(outputXML);

            List<SelectOption> exerciseTypeList = new ArrayList<SelectOption>();

//            exerciseTypeList.add(new SelectOption(" ", ""));
            exerciseTypeList.add(new SelectOption("dtd", "validace DTD"));
            exerciseTypeList.add(new SelectOption("xsl", "XSL Transformace"));
            exerciseTypeList.add(new SelectOption("xpath", "xPath výrazy"));
            exerciseTypeList.add(new SelectOption("schema", "XSD schéma"));
            exerciseTypeList.add(new SelectOption("xquery", "xQuery výrazy"));

            ChoiceRenderer choiceRenderer = new ChoiceRenderer("value", "key");
            exerciseType = new DropDownChoice("exerciseType", new Model(), exerciseTypeList, choiceRenderer);
            exerciseType.add(new DropDownChoiseValidator());
            exerciseType.setLabel(new ResourceModel("form.type"));

            int exerciseTypeIndex = 0;
            if (!getPageParameters().get("exerciseType").isEmpty()) {

                if (getPageParameters().get("exerciseType").toString().equals("dtd")) {
                    exerciseTypeIndex = 0;
                } else if (getPageParameters().get("exerciseType").toString().equals("xsl")) {
                    exerciseTypeIndex = 1;
                } else if (getPageParameters().get("exerciseType").toString().equals("xpath")) {
                    exerciseTypeIndex = 2;
                } else if (getPageParameters().get("exerciseType").toString().equals("schema")) {
                    exerciseTypeIndex = 3;
                } else if (getPageParameters().get("exerciseType").toString().equals("xquery")) {
                    exerciseTypeIndex = 4;
                } else {
                    exerciseTypeIndex = 0;
                }
            }

            if (exerciseTypeIndex != 0) {
                exerciseType.setModelObject(exerciseTypeList.get(1));
            }
            add(exerciseType);

            List<SelectOption> showExpectedResultList = new ArrayList<SelectOption>();

            showExpectedResultList.add(new SelectOption("1", "Ano"));
            showExpectedResultList.add(new SelectOption("0", "Ne"));

            showExpectedResult = new DropDownChoice("showExpectedResult", new Model(), showExpectedResultList, choiceRenderer);
            showExpectedResult.add(new DropDownChoiseValidator());
            showExpectedResult.setLabel(new ResourceModel("form.show"));
            add(showExpectedResult);

            add(new Button("button") {
                @Override
                public void onSubmit() {

                    ExerciseManager exerciseManager = new ExerciseManager();
                    exerciseManager.addExercise((String) task.getModelObject(),
                            (String) taskXML.getModelObject(),
                            (String) inputXML.getModelObject(),
                            (String) expectedResult.getModelObject(),
                            (String) expectedResultXML.getModelObject(),
                            ((SelectOption) exerciseType.getModelObject()).getKey(),
                            ((SelectOption) showExpectedResult.getModelObject()).getKey(),
                            (String) outputXML.getModelObject());

                    info("Nové cvičení bylo úspěšně přidáno.");

                    clearForm();

                }
            });

        }

        private void clearForm() {
            task.setModelObject((String) "uloženo");
            taskXML.setModelObject((String) "uloženo");
            inputXML.setModelObject((String) "uloženo");
            expectedResult.setModelObject((String) "uloženo");
            expectedResultXML.setModelObject((String) "uloženo");
            outputXML.setModelObject((String) "uloženo");

        }

        protected class SelectOption implements java.io.Serializable {

            private String key;
            private String value;

            public SelectOption(String key, String value) {
                this.key = key;
                this.value = value;
            }

            protected String getKey() {
                if (key == null) {
                    return "";
                }
                return key;
            }

            protected void setKey(String key) {
                this.key = key;
            }

            protected String getValue() {
                return value;
            }

            protected void setValue(String value) {
                this.value = value;
            }
        }

        private class NullValidator extends AbstractValidator {

            @Override
            public void onValidate(IValidatable iv) {

                if (iv.getValue() == null || ((String) iv.getValue()).equals("") || iv.getValue().equals("<br>")) {
                    error(iv);
                }

            }

            @Override
            public boolean validateOnNullValue() {
                return true;
            }

            @Override
            protected String resourceKey() {
                return "emptyArea";
            }
        }

        private class NiceEditorVlaidator extends AbstractValidator {

            @Override
            public void onValidate(IValidatable iv) {

                if (iv.getValue().equals("<br>")) {
                    error(iv);
                }

            }

            @Override
            public boolean validateOnNullValue() {
                return true;
            }

            @Override
            protected String resourceKey() {
                return "emptyArea";
            }
        }

        private class DropDownChoiseValidator extends AbstractValidator {

            @Override
            public void onValidate(IValidatable iv) {

                SelectOption so = (SelectOption) iv.getValue();

                if (so == null) {
                    error(iv);
                }

                if (so != null && (so.getValue() == null || so.getValue().equals(""))) {
                    error(iv);
                }

            }

            @Override
            public boolean validateOnNullValue() {
                return true;
            }

            @Override
            protected String resourceKey() {
                return "emptyDropDownChoice";
            }
        }
    }
}
