package cz.muni.fi.wicket;

import cz.muni.fi.user.RegistrationPage;
import org.apache.wicket.Session;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.PropertyModel;

/**
 * Class that represents Panel for Sign
 * 
 * @author Jiří Krejčí
 */
public final class SignPanel extends Panel {

    public SignPanel(String componentName) {
        super(componentName);

        add(new SignInPanel("signInPanel", false) {
            @Override
            public boolean isVisible() {
                SignInSession session = (SignInSession) Session.get();
                return !session.isSignedIn();
            }
        });


        add(new BookmarkablePageLink("registrationLink", RegistrationPage.class)
                .add(new Label("registrationLinkText", "Zaregistrovat se") {
            @Override
            public boolean isVisible() {
                SignInSession session = (SignInSession) Session.get();
                return !session.isSignedIn();
            }
        }));


        add(new Label("signInUser", new PropertyModel(this, "signInUser") {
            @Override
            public Object getObject() {
                SignInSession session = (SignInSession) Session.get();
                return "Přihlášený uživatel: <br />" + session.getUser().getFirstname()
                        + " " + session.getUser().getLastname()
                        + "<br />(" + session.getUser().getRoles().toString() + ")";
            }
        }) {
            @Override
            public boolean isVisible() {
                SignInSession session = (SignInSession) Session.get();
                return session.isSignedIn();
            }
        }.setEscapeModelStrings(false));

        BookmarkablePageLink signOutLink;
        signOutLink = new BookmarkablePageLink("signOutLink", SignOutPage.class);
        signOutLink.add(new Label("signOutLinkText", "Sign out") {
            @Override
            public boolean isVisible() {
                SignInSession session = (SignInSession) Session.get();
                return session.isSignedIn();
            }
        });

        add(signOutLink);
    }
}